@@ -1,7 +1,7 @@
 # -*- perl -*-
 # w3http.pm	--- send http requests, janl's 12" mix for w3mir
 #
-$VERSION=1.0.21;
+$VERSION=1.0.22;
 #
 # This implements http/1.0 requests.  We'll have problems with http/0.9
 # This is in no way specific to w3mir.
@@ -122,6 +122,9 @@ $VERSION=1.0.21;
 #     bkw  17/12/98 -- Fixed problem with tempfile-generation when
 #                      running in forget-mode (-f)
 #     janl 05/01/99 -- Referer: dropped if argument not true -> 1.0.21
+#     janl 13/04/99 -- Added workaround for broken win32 perl resolving.
+#     janl 15/01/00 -- Patch to adapt to URI 1.0 from Takuya Tsumura and
+#                      Andrey A. Chernov
 
 package w3http;
 
@@ -143,6 +146,20 @@ sub URI::URL::_generic::basename {
   $old;
 }
 
+# The URI 1.0 library changed the internal organization a bit
+# Thanks to Andrey A. Chernov for the patch!
+
+sub URI::_generic::basename {
+  my $self = shift;
+  my @p = $self->path_segments;
+  my $old = $p[-1];
+  if (@_) {
+    splice(@p, -1, 1, shift);
+    $self->path_segments(@p)
+  }
+  $old;
+}
+
 
 END { 
   # Remove tmp file and such in here.  That means that main:: gotta catch
@@ -357,6 +374,13 @@ sub query {
   
   # If we're using proxy then set up things...
   print STDERR "\nQUERY:\n",$query,"---\n" if $debug>=2;
+
+  # win32 fix: this should be added in case of troubles with
+  # gethostbyname. possible reason: nameserver down?
+  if ($host =~ /^\d+(\.\d+){3}$/) {
+    # in case gethostbyname will not work ... ;-)
+    $address{$host} = pack 'C4', (split /\./, $host);
+  }
   
   # Find out who to ask, check if we know already
   if (exists($address{$host})) {
@@ -474,17 +498,19 @@ sub query {
     # Check if header is complete
     last if ($header =~ m/(\r?\n\r?\n)/);
   }
+
+  my $orighead = $header;
   
   if (length($header)==0) {
     $restext='the HTTP reply header is empty!';
     return &resetsign;
   }
-  
+
   if ($header =~ m/(\r?\n\r?\n)/) {
-    $header=$`;
-    $document=$';
-  } else {
-    $header=$document;
+    if ($`) {
+      $header=$`;
+      $document=$';
+    }
   }
   
   # Adjust accounting
@@ -496,6 +522,13 @@ sub query {
   # Pick headers to pieces
   ($result,$restext,%headval)=&analyze_header($header);
 
+  if (!$result) {
+    print "\n\nw3mir: BOGUS HTTP REPLY:\n-----\n$header\n-----\n";
+    print "\n\nw3mir: UNPROCESSED REPLY:\n-----\n$orighead\n-----\n";
+    print "\nw3mir: QUERY WAS:\n-----\n$query\n-----\n";
+    die;
+  }
+  
   print STDERR "REPLY:\n",$header,"\n---\n" if $debug>=2;
 
   # Check if the document is a non-encoded text document. The contents
@@ -507,7 +540,7 @@ sub query {
       !defined($headval{'content-encoding'});
   $plaintexthtml=$plaintext && 
     ($headval{'CONTENT-TYPE'} eq 'text/html');
-  
+
   if ($result==200) {
     
     # Save this to a file, or not?  Never save html files.
